@@ -1,9 +1,11 @@
 from __future__ import annotations
 
 import ast
+import contextlib
 import errno
 import fcntl
 import os
+import queue
 import select
 import sys
 import tempfile
@@ -69,13 +71,17 @@ from . import RUNNABLE_KERNEL
 from . import RUNNABLE_SCRIPT
 from .exec import Argument
 from .exec import ArgumentKind
+from .exec import Directory
+from .exec import FileName
+from .exec import FilePath
 from .exec import get_arguments
-from .exec import Directory, FileName, FilePath
 from .gui import IconLabel
 from .kernel import MyKernel
 from .kernel import start_qtconsole
 from .utils import capture
 from .utils import create_code_snippet
+from .utils import create_code_snippet_renderable
+from .utils import is_renderable
 from .utils import select_directory
 from .utils import select_file
 from .utils import stringify_args
@@ -265,20 +271,69 @@ class FunctionRunnableExternalCommand(FunctionRunnable):
 class FunctionRunnableKernel(FunctionRunnable):
     def __init__(self, kernel: MyKernel, func: Callable, args: List, kwargs: Dict, input_queue: Queue):
         super().__init__(func, args, kwargs, input_queue)
-        self._kernel: MyKernel = kernel
+        self.kernel: MyKernel = kernel
         self.startup_timeout = 60  # seconds
         self.client: Optional[KernelClient] = None
+        self.console = Console(record=True, width=120)
 
     def run(self):
 
-        self._kernel = self._kernel or MyKernel()
-
         self.signals.data.emit(f"----- Running script '{self.func_name}' in kernel")
 
         snippet = create_code_snippet(self._func, self._args, self._kwargs)
 
-        if response := self._kernel.run_snippet(snippet):
-            self.signals.data.emit(response)
+        # Flush the IOPub channel before executing the command. This is needed because another
+        # client might be connected that has sent out messages on the pub channel. We want to
+        # catch the output of the given command of course.
+
+        self.kernel.flush()
+
+        self.signals.data.emit("The code snippet:")
+        self.signals.data.emit(create_code_snippet_renderable(self._func, self._args, self._kwargs))
+
+        msg_id = self.kernel.client.execute(snippet, allow_stdin=True)
+
+        while True:
+            try:
+                io_msg = self.kernel.client.get_iopub_msg(timeout=1.0)
+                io_msg_content = io_msg['content']
+
+                if io_msg['msg_type'] == 'stream':
+                    if 'text' in io_msg_content:
+                        text = io_msg_content['text'].rstrip()
+                        self.signals.data.emit(text)
+                elif io_msg['msg_type'] == 'status':
+                    if io_msg_content['execution_state'] == 'idle':
+                        # self.signals.data.emit("Execution State is Idle, terminating...")
+                        break
+                    elif io_msg_content['execution_state'] == 'busy':
+                        # self.signals.data.emit("Execution State is Busy, starting...")
+                        continue
+                # elif io_msg['msg_type'] == 'execute_input':
+                #     self.signals.data.emit("The code snippet:")
+                #     source_code = io_msg_content['code']
+                #     syntax = Syntax(source_code, "python", theme='default')
+                #     self.signals.data.emit(syntax)
+
+            except queue.Empty:
+                with contextlib.suppress(queue.Empty):
+                    in_msg = self.kernel.client.get_stdin_msg(timeout=0.1)
+
+                    if in_msg['msg_type'] == 'input_request':
+                        prompt = in_msg['content']['prompt']
+                        if self._check_for_input and any(pattern in prompt for pattern in self._input_patterns):
+                            self.signals.data.emit(prompt)
+                            response = self.handle_input_request(prompt)
+                            self.kernel.client.input(response)
+
+        msg = self.kernel.client.get_shell_msg(msg_id)
+        if msg['msg_type'] == "execute_reply":
+            status = msg['content']['status']
+            self.signals.data.emit(f"{status = }")
+            if status == 'error' and 'traceback' in msg['content']:
+                traceback = msg['content']['traceback']
+                # self.signals.data.emit(remove_ansi_escape('\n'.join(traceback)))
+                self.signals.data.emit(Text.from_ansi('\n'.join(traceback)))
 
         self.signals.finished.emit(self, self.func_name, True)
 
@@ -402,7 +457,7 @@ class SourceCodeWindow(QWidget):
         text_edit = QTextEdit()
 
         console = Console(record=True, width=1200)
-        syntax = Syntax(source_code, "python", theme='default')
+        syntax = Syntax(source_code, "python", theme='default', line_numbers=True)
 
         with console.capture():
             console.print(syntax)
@@ -462,6 +517,16 @@ class DynamicButton(QWidget):
 
         self.label_icon = IconLabel(icon_path=self.icon_path, size=icon_size)
         label_text = QLabel(self.function_display_name)
+        if self._function.__ui_immediate_run__:
+            # This style will draw a 2 pixel horizontal line under the label
+            label_text.setStyleSheet(textwrap.dedent(
+                """\
+                    padding-bottom: 0px; 
+                    border-bottom-width: 2px; 
+                    border-bottom-style: solid; 
+                    border-radius: 0px;
+                """)
+            )
 
         layout.addWidget(self.label_icon)
         layout.addSpacing(self.horizontal_spacing)
@@ -510,12 +575,21 @@ class DynamicButton(QWidget):
 
     @property
     def function_display_name(self) -> str:
-        return self._function.__ui_display_name__ or self.label or self._function.__name__
+        name = self._function.__ui_display_name__ or self.label or self._function.__name__
+
+        # The following line will put the display_name within triangles: ▶︎ name ◀︎
+        # when the immediate_run flag is True
+        # name = f"\u25B6 {name} \u25C0" if self._function.__ui_immediate_run__ else name
+
+        return name
 
     @property
     def label(self) -> str:
         return self._label
 
+    def immediate_run(self):
+        return self.function.__ui_immediate_run__
+
     def __repr__(self):
         return f"DynamicButton(\"{self.label}\", {self.function})"
 
@@ -968,6 +1042,23 @@ class View(QMainWindow):
 
     def the_button_was_clicked(self, button: DynamicButton, *args, **kwargs):
 
+        if button.immediate_run():
+            self.run_function(button.function, [], {}, RUNNABLE_SCRIPT)
+
+            # Remove any existing arguments panel from a previous button
+
+            if self._args_panel:
+                self._args_panel.hide()
+                self._args_panel = None
+
+            # Deselect the previously selected button
+
+            if self.previous_selected_button is not None:
+                self.previous_selected_button.deselect()
+                self.previous_selected_button = None
+
+            return
+
         # TODO
         #   * This should be done from the control or model and probably in the background?
         #   * Add ArgumentsPanel in a tabbed widget? When should it be removed from the tabbed widget? ...
@@ -1002,7 +1093,7 @@ class View(QMainWindow):
 
     @pyqtSlot(object)
     def function_output(self, data: object):
-        self._console_panel.append(str(data))
+        self._console_panel.append(data if is_renderable(data) else str(data))
 
     @pyqtSlot(object, str, bool)
     def function_complete(self, runnable: FunctionRunnable, name: str, success: bool):
@@ -1025,6 +1116,7 @@ class View(QMainWindow):
 
     @pyqtSlot(str)
     def input_request(self, msg: str):
+        # The argument msg contains the last lines of the output on which the input request was recognized.
         button = QMessageBox.question(
             self,
             "Input Request from Script",
